import { ITodo } from "./interfaces/ITodo";
import { RabbitMQ } from "./rabbitmq/RabbitMQ";
import { MongoDbModel } from "./mongodb/MongoDb";
import { DateService } from "./services/DateService";

export class NotificationService {
  rabbitmq: RabbitMQ;
  mongoModel: MongoDbModel;
  currentDate: DateService;

  constructor() {
    this.currentDate = DateService.getInstance();
    this.rabbitmq = new RabbitMQ();
    this.mongoModel = new MongoDbModel();

    this.startListener();
  }

  async startListener() {
    if (this.IsUserConnected()) {
      await this.rabbitmq.connect();
      this.rabbitmq.startConsumer(this.newMessageValidator.bind(this));
    } else {
      // Retry in 5 seconds
      setTimeout(() => {
        console.log("Waiting for user to connect...")
        this.startListener();
      }, 5000);
    }
  }

  async newMessageValidator(message: ITodo) {
    try {
      const todo: ITodo | void = await this.mongoModel.getTodoById(message._id.toString());
      if (todo) {
        const due_date = new Date(todo.due_date);
        if (todo.status === "pending" && due_date > this.currentDate.getCurrentDate()) {
          await this.mongoModel.updateTodoStatus(todo);
          await this.sendNotification(todo); // Send notification to user
        }
      } else {
        console.error("todo Not found");
      }
    } catch (error) {
      console.error(error.message);
    }
  }

  private IsUserConnected() {
    return true;
  }

  private async sendNotification(todo: ITodo) {
    console.log("Sending notification for Todo:", todo._id);
  }
}

new NotificationService();
