import { MongoClient, ObjectId } from "mongodb";
import { ITodo } from "../interfaces/ITodo";
import { EnvService } from "../services/EnvService";

export class MongoDbModel {
  client: MongoClient;
  envSerivce: EnvService;

  constructor() {
    this.envSerivce = EnvService.getInstance();
    this.client = new MongoClient(
      this.envSerivce.getEnvVariable("DATABASE_URL")
    );
  }

  public updateTodoStatus = async (todo: ITodo) => {
    try {
      await this.client.connect();
      const db = this.client.db(
        this.envSerivce.getEnvVariable("MONGO_DB_NAME")
      );
      const todosCollection = db.collection("todos");
      const result = await todosCollection.updateOne(
        { _id: new ObjectId(todo._id) },
        { $set: { status: "completed" } }
      );
    } catch (error) {
      console.error("Error updating Todo status:", error);
    } finally {
      await this.client.close();
    }
  };

  public getTodoById = async (id: string): Promise<ITodo | void> => {
    try {
      await this.client.connect();
      const db = this.client.db(
        this.envSerivce.getEnvVariable("MONGO_DB_NAME")
      );
      const todosCollection = db.collection("todos");
      const todo = await todosCollection.findOne({
        _id: new ObjectId(id),
      });

      // Convert the document to ITodo type
      if (todo) {
        const convertedTodo: ITodo = {
          _id: new ObjectId(todo._id),
          title: todo.title,
          description: todo.description,
          due_date: todo.due_date,
          createAt: todo.createAt,
          updateAt: todo.updateAt,
          status: todo.status,
        };

        return convertedTodo;
      }

      return undefined;
    } catch (error) {
      console.error("Error getting Todo by id:", error);
    } finally {
      await this.client.close();
    }
  };
}
