import { Router } from "express";
import { TodoController } from "../controllers/todoController";
import {
  createTodoMiddleWare,
  paramIdMiddleware,
} from "../middleware/createTodoMiddleWare";
import { DateService } from "../services/DateService";

class TodoRouter {
  router: Router;
  todoController: TodoController;
  currentDate: DateService;

  constructor() {
    this.currentDate = DateService.getInstance();
    this.router = Router();
    this.todoController = new TodoController();
    this.setRoutes();
  }

  private setRoutes() {
    this.router.get("/", this.todoController.getAll);
    this.router.get("/:id", paramIdMiddleware, this.todoController.getOne);
    this.router.post("/", createTodoMiddleWare, this.todoController.createOne);
    this.router.put(
      "/:id",
      createTodoMiddleWare,
      this.todoController.updateOne
    );
    this.router.delete(
      "/:id",
      paramIdMiddleware,
      this.todoController.deleteOne
    );
    this.router.delete("/", this.todoController.removeAll);
  }

  public getRouter() {
    return this.router;
  }
}

export default new TodoRouter().getRouter();
